<?php
/**
 * Roman.Szablatura RON Software 13.09.2019
 */

namespace RonPortal\Jidelna;


use Exception;
use Ron\Controls\BrowserField;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\Http\Csrf;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterItem;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniUtils;


class VybratPolozkuJidelnickuField extends BrowserField
{
    /** @var PravaNaOsoby */
    private $pravo;

    public function __construct($pravo)
    {
        parent::__construct();
        $this ->pravo = $pravo;
        $this ->setHeader(
            '<input type="checkbox" id="vybrat_vsechny_radky" class="setValue" data-selector=".vybrat-radek" data-value-checked="1" data-value-unchecked="0"/>'
        );
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$rowData['col-datumcasvydani']  &&  !$rowData['col-datumcaszruseni']  &&  $this ->pravo ->lzeZobrazitPP($rowData['col-idpp'])) {
            return '<input type="checkbox" class="vybrat-radek" />';
        }

        return '';
    }
}


/**
 * Class OdstranitPolozkuJidelnicku
 */
class OdstranitPolozkuJidelnicku extends BrowserField
{
    /** @var PravaNaOsoby $pravoZruseni */
    public $pravoZruseni;


    /**
     * DetailPolozkyJidelnicku constructor.
     * @param Database $database
     * @param PravaNaOsoby $pravoZruseni
     */
    public function __construct($database, $pravoZruseni)
    {
        parent::__construct($database);
        $this->pravoZruseni = $pravoZruseni;
        $this->code = 'col-zrusit';
    }


    function getHtmlTitle($rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-datumcasvydani']) {
            return TextManager::_('Jidelna_objednavka_vydana');
        }

        return parent::getHtmlTitle($rowNumber, $recordId, $rowData);
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        //Pokud ma nastaveno datum vydani pak nezobrazit odkaz
        if ($rowData['col-datumcasvydani']  ||  $rowData['col-datumcaszruseni']) {
            return '';
        }

        if ($this->pravoZruseni->lzeZobrazitPP($rowData['col-idpp'])) {
            return
                "<i class='fa inline-glyph glyph-button glyph-delete cursor-pointer zrusit-polozku-jidelnicku'" .
                " title='" . TextManager::_('jidelna_objednavku_odstranit') . "'" .
                '>' .
                "</i>";
//            return
//                "<span class='rectangle confirmQuestion js zrusit-polozku-jidelnicku'" .
//                " data-ron-confirm-text='$confirmText'" .
//                " data-ron-href='?menuid=jidelnaobjednavky&action=objednavkuodstranitodkazem&osoba={$rowData['col-idpp']}&id=$rowData[1]'" .
//                " data-ron-working-info='" . TextManager::_('Jidelna_odstranuji_objednavku') . "'>" .
//                TextManager::_('jidelna_objednavku_odstranit') .
//                "</span>";
        }

        return '';
    }
}


/**
 * Class PrevestPolozkuJidelnicku
 */
class PrevestPolozkuJidelnicku extends BrowserField
{
    /** @var PravaNaOsoby $pravo */
    public $pravoPrevedeni;


    /**
     * PrevestPolozkuJidelnicku constructor.
     * @param Database $database
     * @param PravaNaOsoby $pravoPrevedeni
     */
    public function __construct($database, $pravoPrevedeni)
    {
        parent::__construct($database);
        $this->pravoPrevedeni = $pravoPrevedeni;
    }


    function getHtmlTitle($rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-datumcasvydani']) {
            return TextManager::_('Jidelna_objednavka_vydana');
        }

        return parent::getHtmlTitle($rowNumber, $recordId, $rowData);
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        //Pokud ma nastaveno datum vydani pak nezobrazit odkaz
        if ($rowData['col-datumcasvydani']  ||  $rowData['col-datumcaszruseni']) {
            return '';
        }

        if ($this->pravoPrevedeni->lzeZobrazitRC($rowData[2])) {
            //Momentalne nelze pripojit poznamku prevedeni (neni vyreseno datove)
            return
                "<a href='#' class='objednavku-prevest glyph-button cursor-pointer'" .
                //" title='" . TextManager::_('Jidelna_objednavku_prevest') . "'" .
                " data-ron-akce='prevestobjednavku'" .
                " data-ron-idpp='{$rowData['col-idpp']}'" .
                " data-ron-id-polozky-jidelnicku='$rowData[1]'" .
                " data-ron-jidelna='" . StringUtils::htmlTextConvert($rowData['col-jidelna']) . "'" .
                " data-ron-jidelna-popis='" . StringUtils::htmlTextConvert($rowData['col-jidelna-popis']) . "'" .
                " data-ron-jmeno='" . StringUtils::htmlTextConvert($rowData['@col-osoba']) . "'" .
                " data-ron-popis-objednavky='" . StringUtils::htmlTextConvert($rowData['col-popis']) . "'" .
                " data-ron-datum-objednavky='" . StringUtils::format('date', $this ->database ->stringToTimestamp($rowData['col-datum'])) . "'" .
                " data-ron-cislo-objednavky='$rowData[0]'>" .
                //"<i class='inline-glyph fa glyph-exchange'></i>" .
                TextManager::_('Jidelna_objednavku_prevest') .
                "</a>";
        }

        return '';
    }
}


/**
 * Class SeznamObjednavekJidlaBrowser
 */
class SeznamObjednavekBrowser extends RonPortalBrowser
{

    /** @var PravaNaOsoby $pravoPrevedeni */
    public $pravoPrevedeni;

    /** @var PravaNaOsoby $pravoZruseni */
    public $pravoZruseni;

    private $browserAction = '';
    /** @var PravaNaOsoby */
    private $browserAction_Pravo = null;

    /**
     * SeznamObjednavekJidlaBrowser constructor.
     * @param Database $database
     * @param $pravoPrevedeni
     * @param $pravoZruseni
     * @throws Exception
     */
    public function __construct($database)
    {
        parent::__construct($database, 'jidelnaobjednavky');

        $this->title = TextManager::_('Jidelna_seznam_objednavek');
        $this->code = 'browserjidelnaobjednavky';
        $this->tableId = 'browserjidelnaobjednavky';
        $this->browserUrl = '?menuid=jidelnaobjednavky';

        $this->pravoPrevedeni = Prava::vratPrava('jidelna_prevedeni_objednavky');
        $this->pravoZruseni = Prava::vratPrava('jidelna_zruseni_objednavky');

        switch ($_GET['browseraction'] ?? '') {
            case 'zruseni' :
                $this ->browserAction_Pravo = $this ->pravoZruseni;
                break;

            case 'prevedeni' :
                $this ->browserAction_Pravo = $this ->pravoPrevedeni;
                break;
        }

        if ($this ->browserAction_Pravo  &&  $this ->browserAction_Pravo ->jsouNejakePrava()) {
            $this ->browserAction = $_GET['browseraction'];
        }

        $this->sqlSelectPart =
            "OJ.ID, OJ.IDPolozkyJidelnicku, OJ.RC";               //0..2

        $this ->primaryField = 0;

        $this->sqlFromPart =
            'ObjednaneJidlo AS OJ' .
            ' JOIN PracovniPomer ON (PracovniPomer.RC = OJ.RC) AND (PracovniPomer.OsobniCislo = OJ.OsobniCislo) AND (PracovniPomer.DatumNastupu = OJ.DatumNastupu)' .
            ' JOIN Osoba ON (Osoba.RC = PracovniPomer.RC)' .
            ' JOIN PolozkaJidelnicku PJ ON (PJ.ID = OJ.IDPolozkyJidelnicku)' .
            ' JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = 8 AND Rozdeleni.Kod = PJ.jidelna)' .
            ' JOIN TypDennihoJidla ON (TypDennihoJidla.Kod = PJ.typDennihoJidla)';

        $this ->tableHtmlParams['data-tabulky'] = 'ObjednaneJidlo OJ;PracovniPomer;Osoba;PolozkaJidelnicku PJ;Rozdeleni;TypDennihoJidla';


        if ($this ->browserAction_Pravo  &&  $this ->browserAction_Pravo ->jsouNejakePrava()) {
            $this->addField(new VybratPolozkuJidelnickuField($this->browserAction_Pravo));
        }

        $this->addField()
            ->setSqlFieldName('PJ.Den') ->setHeaderText('Den') ->setOrdering(true)
            ->setFormat('finedate') ->setAlign('center')
            ->setCode('col-datum')
            ->setSortSQL('PJ.Den, Osoba.Prijmeni, Osoba.Jmeno', 'PJ.Den DESC, Osoba.Prijmeni DESC, Osoba.Jmeno DESC');

        $this->defaultSort = 'col-datum';

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Jmeno')
            ->setCode('col-jmeno');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Prijmeni')
            ->setCode('col-prijmeni');

        $this ->addField(new JmenoPrijmeniBrowserField($this ->database, 'col-jmeno', 'col-prijmeni'))
            ->setHeaderText('Osoba') ->setClassName('jmeno-stravnika')
            ->setCode('col-osoba')
            ->setOrdering(true) ->setSortSQL('Osoba.Prijmeni, Osoba.Jmeno, PJ.Den')
            ->setSortSQLDesc('Osoba.Prijmeni DESC, Osoba.Jmeno DESC, PJ.Den DESC');

        $this ->addField()
            ->setSqlFieldName('OJ.OsobniCislo') ->setHeaderText('Osobni_cislo') ->setOrdering(true)
            ->setCode('col-osobnicislo');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OJ.DatumCasObjednani') ->setFormat('finedatetime')
            ->setCode('col-datumcasobjednani');

        $this->addField()
            ->setSqlFieldName('OJ.ID', 0) ->setOrdering(true) ->setHeaderText('Jidelna_ID_polozky')
            ->setCode('col-id')
            ->setTitleFieldIndex('@col-datumcasobjednani') ->setTitlePrefix(TextManager::_('Jidelna_datum_cas_objednani') . ': ')
            ->setVisible(false);

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PJ.CisloNabidky')
            ->setCode('col-cislo-nabidky');

        $this->addField()
            ->setSqlFieldName('TypDennihoJidla.Popis') ->setOrdering(true)
            ->setHeaderText('Jidelna_typ_jidla')
            ->setCode('col-typjidla')
            ->setSqlFieldIndex2('col-cislo-nabidky', ' ')
            ->setSortSQL('TypDennihoJidla.Popis, PJ.CisloNabidky', 'TypDennihoJidla.Popis DESC, PJ.CisloNabidky DESC')
            ;

        $this->addField()
            ->setSqlFieldName('Rozdeleni.Popis') ->setOrdering(true)
            ->setHeaderText('Jidelna')
            ->setCode('col-jidelna-popis');

        $this->addField()
            ->setSqlFieldName('PJ.PopisNabidky') ->setHeaderText('Detail')
            ->setCode('col-popis')
            ->setLink('?menuid=jidelnaobjednavky&action=detailobjednavkyjidla&id=')
            ->setLinkAttributes(['class' => 'detail-objednavky']);

        $this->addField(new OdstranitPolozkuJidelnicku($database, $this->pravoZruseni));

        $this->addField(new PrevestPolozkuJidelnicku($database, $this->pravoPrevedeni))
            ->setCode('col-prevest');

        $this ->addField()
            ->setSqlFieldName('OJ.DatumCasVydani') ->setFormat('finedatetime')
            ->setCode('col-datumcasvydani')
            ->setHeaderText('Jidelna_vydano') ->setAlign('right');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OJ.Uzivatel')
            ->setCode('col-uzivatel');

        $this ->addField()
            ->setSqlFieldName('OJ.DatumCasObjednani') ->setFormat('finedatetime')
            ->setCode('col-datumcasobjednani')
            ->setHeaderText('Jidelna_cas_objednani') ->setAlign('right')
            ->setTitleFieldIndex('col-uzivatel');

        if ( RonPortalSettings::getIntValue(RonPortalSettings::BURZA_POVOLENA) ) {
            $this ->addField()
                ->setSqlFieldName('OJ.DatumCasVlozeniNaBurzu') ->setFormat('finedatetime')
                ->setCode('col-datumcasvlozeninaburzu')
                ->setHeaderText('Jidelna_cas_vlozeni_na_burzu') ->setAlign('right');
        }

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OJ.DatumCasZruseni') ->setFormat('finedatetime')
            ->setCode('col-datumcaszruseni')
            ->setHeaderText('Jidelna_zruseno') ->setAlign('right');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PracovniPomer.Id')
            ->setCode('col-idpp');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PJ.Jidelna')
            ->setCode('col-jidelna');
    }


    function setFilterForm()
    {
        $this ->filterFormAppCode = null; // nezobrazovat obdobi
        $this ->filterFormUseGet = true;
        $this ->filterFormRights = 'prohl_jidelna';
        $this ->filterFormVcetneExternich = true;

        parent::setFilterForm();

        $this ->filterForm ->nastavDatumOdDo(true, true);
        $this ->filterForm ->datumOdDoLinks = ['today', 'currentMonth'];
        $this ->filterForm ->povolitVsechyOsoby = true;

        // typ denniho jidla
        $filterTyp = new FilterFormFilter('typjidla', FilterFormFilter::TYPE_OPTIONS);
        $filterTyp
            ->setCaption(TextManager::_('Jidelna_pouze_typ_jidla'))
            ->addItem('', TextManager::_('_vse_'))
            ->setOptionsSql(
                'SELECT Kod, Popis FROM TypDennihoJidla WHERE %FILTER% ORDER BY %ORDER%',
                'TypDennihoJidla.Aktivni = 1', 'Popis')
            ->setSqlFilter('PJ.typDennihoJidla = ')
            ;

        $this ->filterForm ->addFilter($filterTyp);

        // jidelna
        $filterJidelna = new FilterFormFilter('jidelna');

        $params = $this ->database ->createParameters();
        $filterJidelna
            ->setCaption(TextManager::_('Jidelna'))
            ->addItem('', TextManager::_('Vse'))
            ->setOptionsSql(new SqlStatement(
                'SELECT Kod, Popis FROM Rozdeleni' .
                ' WHERE KodTypuRozdeleni = 8' .
                ' AND %FILTER%' .
                ' AND DatumVytvoreni <= ' . $params ->addDateParameter(time()) . // TODO - podle vybraneho obdobi
                ' AND (DatumZruseni >= ' . $params ->addDateParameter(time()) . ' OR DatumZruseni IS NULL)' .
                ' ORDER BY %ORDER%',
                $params),
                '1 = 1', 'Popis')
            ->setSqlFilter('PJ.Jidelna = ');

        $this ->filterForm ->addFilter($filterJidelna);

        // vydano
        $filtrVydano = new FilterFormFilter('vydano');
        $filtrVydano
            ->setItems([
                (new FilterFormFilterItem('0', TextManager::_('Vse'))) ->setSqlFilter('DatumCasZruseni IS NULL'),
                (new FilterFormFilterItem('1', TextManager::_('Jen_nevydane_objednavky'))) ->setSqlFilter('DatumCasZruseni IS NULL  AND  DatumCasVydani IS NULL'),
                (new FilterFormFilterItem('2', TextManager::_('Jen_vydane_objednavky'))) ->setSqlFilter('DatumCasZruseni IS NULL  AND  DatumCasVydani IS NOT NULL'),
                (new FilterFormFilterItem('3', TextManager::_('Jen_zrusene_objednavky'))) ->setSqlFilter('DatumCasZruseni IS NOT NULL')
            ])
            ->setDefaultValue('0');
        $this ->filterForm ->addFilter($filtrVydano);

        // akce pro hromadne operace
        $this ->filterForm ->addFilter(
            (new FilterFormFilter('browseraction', FilterFormFilter::TYPE_HIDDEN))
            ->setHtmlAttributes(['id' => 'objednavky_browseraction'])
            ->setSessionName('') // neukladat do session
        );
    }


    /**
     * @return SqlStatement|string
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        $result .=
            ($result ? ' AND ' : '') .
            ' PJ.Den >= ' . $params->addDateParameter($this ->filterForm ->datumOd) .
            ' AND PJ.Den <= ' . $params->addDateParameter($this ->filterForm ->datumDo) .
            "\n";

        if (!$this ->filterForm ->vsechnyOsoby) {
            $result .=
                ' AND PracovniPomer.Id = ' . $params->addParameter($this ->filterForm ->vybranaOsoba) .
                "\n";
        } else {
            if ($this ->filterForm ->kodRozdeleni != '') {
                $result .=
                    ' AND ' .
                    RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'PracovniPomer',
                        $this ->filterForm ->kodTypuRozdeleni, $this ->filterForm ->kodRozdeleni,
                        time(), time(), $this ->filterForm ->vcetnePodrizenychRozdeleni, $params) .
                    "\n";
            }

            if ($this ->filterForm ->kodRozdeleni2 != '') {
                $result .=
                    ' AND ' .
                    RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'PracovniPomer',
                        $this ->filterForm ->kodTypuRozdeleni2, $this ->filterForm ->kodRozdeleni2,
                        time(), time(), $this ->filterForm ->vcetnePodrizenychRozdeleni2, $params) .
                    "\n";
            }
        }

        //$filtrPrava = Prava::sqlFiltrPodlePrav([$this ->pravoPrevedeni, $this ->pravoZruseni, Prava::vratPrava('prohl_jidelna')], 'PracovniPomer');
        $filtrPrava = Prava::sqlFiltrPodlePrav(Prava::vratPrava('prohl_jidelna'), 'PracovniPomer');
        if ($filtrPrava) {
            $result .= ' AND (' . $filtrPrava . ')' . "\n";
        }

        return new SqlStatement($result, $params);
    }


    /**
     * Za filtry zobrazi tlacitka pro hromadne operace
     * @return void
     */
    function renderFilter()
    {
        parent::renderFilter();
        return;

        echo '<div id="browserjidelnaobjednavky_buttons" class="margin-top-2">';
        if ($this ->browserAction) {
            echo
                // tlacitko pro provedeni akce
                "<button class='submitbutton' id='button_provest_'{$this ->browserAction}>" .
                TextManager::_('Jidelna_provest_' . $this ->browserAction) .
                '</button>' .
                '<span class="margin-all-4"> </span>' .
                // tlacitko storno
                '<button class="button setValue" data-value="" data-selector="#objednavky_browseraction" data-submit="1">' .
                TextManager::_('Cancel') .
                '</button>';
        } else {
            // hromadne zrusit
            if ($this->pravoZruseni->jsouNejakePrava()) {
                echo
                    '<button class="button setValue" id="button_hromadnezruseni" data-value="zruseni" data-selector="#objednavky_browseraction" data-submit="1">' .
                    TextManager::_('Jidelna_hromadne_zruseni') .
                    '</button>';
            }
        }
        echo '</div>';
    }



    public function getRowAttributes($values, $className, $rowNumber)
    {
        return
            'data-id-objednavky="' . $values['col-id'] . '" ' .
            'data-csrfkey="' . Csrf::getCsrfKey('polozkajidelnicku_' . $values['col-id']) . '" ' .
            parent::getRowAttributes($values, ($values['col-datumcaszruseni'] ? 'zrusena-objednavka ' : '') . $className, $rowNumber);
    }


}
